@@ -1,16 +1,18 @@
 import json
 from datetime import datetime, timedelta
-from typing import Any, Dict, List, Optional, Union
+from typing import Any, Dict, Optional, Union
 
 from pyspark.sql import DataFrame
 from pyspark.sql.functions import lit
 from pyspark.sql.types import StructType
 
 from ng_data_pipelines_sdk.aws_interface import AWSInterface
+from ng_data_pipelines_sdk.custom_logger import logger
 from ng_data_pipelines_sdk.interfaces import (
     DATE_FORMATTER,
-    DataFrameReadWriteParams,
+    DataFrameDict,
     DatePartition,
+    EmptyDataFrameException,
     FileType,
     FnKind,
     InputDataFrameParams,
@@ -18,16 +20,15 @@ from ng_data_pipelines_sdk.interfaces import (
     OutputDataFrameParams,
     OutputDataFrameParamsDict,
     S3BucketParams,
-    S3ReadJsonParams,
+    S3ReadSchemaParams,
     StepParams,
     StepParamsDict,
     TransformParams,
     TransformParamsDict,
-    DataFrameDict,
 )
-from ng_data_pipelines_sdk.custom_logger import logger
 from ng_data_pipelines_sdk.spark_manager import SparkManager
 from ng_data_pipelines_sdk.utils import (
+    generate_date_paths,
     handle_pyspark_timestamp_in_schema,
 )
 
@@ -61,33 +62,6 @@ class DataFrameManager:
     def _get_bucket_url(self, bucket_params: S3BucketParams) -> str:
         return f"{self.file_system}://{bucket_params.bucket_name}"
 
-    def _get_processing_date_partition_path(
-        self, df_params: Union[DataFrameReadWriteParams, DataFrameReadWriteParams]
-    ) -> str:
-        if df_params.processing_date_partitions is None:
-            return ""
-
-        if not isinstance(df_params.processing_date, datetime):
-            raise ValueError(
-                f"processing_date must be a datetime object to calculate partition path. Received: '{df_params.processing_date}'"
-            )
-
-        partition_path_segments = [
-            f"{date_part.value}={DATE_FORMATTER[date_part.value.replace('part_', '')](df_params.processing_date)}"
-            for date_part in df_params.processing_date_partitions
-        ]
-
-        processing_date_partition_path = "/".join(partition_path_segments)
-
-        return processing_date_partition_path
-
-    def _get_dataframe_path_with_date_partitions(
-        self, df_params: DataFrameReadWriteParams
-    ) -> str:
-        dataframe_path_with_date_partitions = f"{df_params.path_to_dataframe_root}/{self._get_processing_date_partition_path(df_params)}"
-
-        return dataframe_path_with_date_partitions.strip("/")
-
     def convert_schema_object_to_pyspark_schema(
         self, schema_object: Dict[str, Any]
     ) -> StructType:
@@ -99,7 +73,7 @@ class DataFrameManager:
         return pyspark_schema
 
     def retrieve_schema_from_s3(
-        self, s3_read_schema_params: S3ReadJsonParams
+        self, s3_read_schema_params: S3ReadSchemaParams
     ) -> StructType:
         """
         Retrieve the schema for the data from an AWS S3 bucket.
@@ -112,12 +86,10 @@ class DataFrameManager:
         """
 
         if s3_read_schema_params is None:
-            raise ValueError(
-                "s3_schema_path_params must be provided to retrieve schema from S3"
-            )
+            raise ValueError("s3_schema_path_params must be provided")
 
         if s3_read_schema_params.bucket_params.bucket_name is None:
-            raise ValueError("bucket_name must be provided to retrieve schema from S3")
+            raise ValueError("bucket_name must be provided")
 
         bucket_name = s3_read_schema_params.bucket_params.bucket_name
         path_to_file = s3_read_schema_params.path
@@ -152,36 +124,81 @@ class DataFrameManager:
             logger.error(error_message)
             raise ValueError(error_message)
 
-    def read_dataframe(self, df_params: InputDataFrameParams) -> DataFrame:
-        read_params = df_params.df_params
-        bucket_url = self._get_bucket_url(read_params.bucket_params)
-        if read_params.specific_path:
-            path = f"{bucket_url}/{read_params.specific_path}"
-        else:
-            path = f"{bucket_url}/{self._get_dataframe_path_with_date_partitions(read_params)}"
+    def read_dataframe(
+        self,
+        input_df_params: InputDataFrameParams,
+        cache: bool = False,
+        verbose: bool = True,
+    ) -> DataFrame:
+        """
+        Reads a DataFrame from a specified location.
 
-        if df_params.pyspark_schema_struct:
-            schema = self.convert_schema_object_to_pyspark_schema(
-                schema_object=df_params.pyspark_schema_struct
+        Args:
+            df_params (InputDataFrameParams): The parameters for reading the DataFrame.
+
+        Returns:
+            DataFrame: The read DataFrame.
+
+        """
+        bucket_url = self._get_bucket_url(input_df_params.dataframe_bucket_params)
+        full_base_path = f"{bucket_url}/{input_df_params.dataframe_base_path}"
+
+        if input_df_params.dataframe_specific_paths:
+            if isinstance(input_df_params.dataframe_specific_paths, list):
+                paths = [
+                    f"{bucket_url}/{path}"
+                    for path in input_df_params.dataframe_specific_paths
+                ]
+            else:
+                paths = f"{bucket_url}/{input_df_params.dataframe_specific_paths}"
+        elif input_df_params.read_date_params:
+            date_partitions = input_df_params.read_date_params.date_partitions
+            paths = generate_date_paths(
+                base_path=f"{bucket_url}/{input_df_params.dataframe_base_path}",
+                dates=input_df_params.read_date_params.read_dates,
+                year_partition=date_partitions.get(DatePartition.YEAR, "year"),
+                month_partition=date_partitions.get(DatePartition.MONTH, "month"),
+                day_partition=date_partitions.get(DatePartition.DAY, "day"),
             )
-        elif df_params.s3_schema_path_params:
-            schema = self.retrieve_schema_from_s3(df_params.s3_schema_path_params)
+
+            # Prevent pyspark from adding date columns to the DataFrame
+            if not input_df_params.read_date_params.add_all_date_columns:
+                full_base_path = None
+
+        else:
+            paths = f"{bucket_url}/{input_df_params.dataframe_base_path}"
+
+        if input_df_params.pyspark_schema_struct:
+            schema = self.convert_schema_object_to_pyspark_schema(
+                schema_object=input_df_params.pyspark_schema_struct
+            )
+        elif input_df_params.s3_schema_path_params:
+            schema = self.retrieve_schema_from_s3(input_df_params.s3_schema_path_params)
         else:
             schema = None
 
-        logger.info(f"Reading DataFrame from {path}...")
+        if verbose:
+            logger.info("Starting read operation.")
 
         df = self.spark_manager.read(
-            env=read_params.bucket_params.env,
-            path=path,
-            file_type=df_params.df_params.file_type,
+            env=input_df_params.dataframe_bucket_params.env,
+            paths=paths,
+            file_type=input_df_params.dataframe_file_type,
+            base_path=full_base_path,
             schema=schema,
+            cache=cache,
         )
 
+        if verbose:
+            logger.info("Read operation completed successfully.\n")
+
+        if df is None:
+            raise EmptyDataFrameException
+
         return df
 
     def write_schema(
-        self, df: DataFrame, write_params: DataFrameReadWriteParams
+        self, df: DataFrame, output_df_params: OutputDataFrameParams
     ) -> None:
         """
         Write the schema of the DataFrame to the specified location.
@@ -193,155 +210,171 @@ class DataFrameManager:
         - ValueError: If there is an issue with writing the schema object to AWS S3.
         """
         pyspark_schema_json = df.schema.jsonValue()
-        bucket_name = write_params.bucket_params.bucket_name
 
-        if bucket_name is None:
+        if output_df_params.dataframe_bucket_params.bucket_name is None:
             raise ValueError("bucket_name must be provided to write schema to S3")
 
-        if write_params.specific_path:
+        if output_df_params.dataframe_specific_path:
+            if isinstance(output_df_params.dataframe_specific_path, list):
+                raise ValueError(
+                    "Specific path is set as a list. Only a single path is allowed for writing the schema."
+                )
             logger.warning(
                 "Specific path is set for writing the schema. Schema will be written to the same path."
             )
-            path_to_schema = write_params.specific_path
+            path_to_schema = output_df_params.dataframe_specific_path
         else:
-            if write_params.path_to_dataframe_root is None:
+            if output_df_params.dataframe_base_path is None:
                 raise ValueError(
-                    "Neither 'specific_path' nor 'path_to_dataframe_root' is set for writing the schema."
+                    "Neither 'specific_path' nor 'dataframe_base_path' is set for writing the schema."
                 )
 
-            parent_folder_of_dataframe_root = "/".join(
-                write_params.path_to_dataframe_root.split("/")[:-1]
+            parent_folder_of_dataframe_base_path = "/".join(
+                output_df_params.dataframe_base_path.split("/")[:-1]
             )
-            path_to_schema = f"{parent_folder_of_dataframe_root}/schema.json"
+            path_to_schema = f"{parent_folder_of_dataframe_base_path}/schema.json"
 
         try:
             self.aws_interface.put_object_aws(
-                env=write_params.bucket_params.env,
-                bucket_name=bucket_name,
+                env=output_df_params.dataframe_bucket_params.env,
+                bucket_name=output_df_params.dataframe_bucket_params.bucket_name,
                 object_name=path_to_schema,
                 object_data=pyspark_schema_json,
             )
         except Exception as e:
             raise ValueError(
-                f"Error writing schema to {bucket_name}/{path_to_schema}: {e}"
+                f"Error writing schema to {output_df_params.dataframe_bucket_params.bucket_name}/{path_to_schema}: {e}"
             )
 
-    def _add_date_partitions_columns(
-        self, df: DataFrame, date_partitions: DatePartition
-    ) -> DataFrame:
-        for date_part, value in date_partitions:
-            if value is not None:
-                df = df.withColumn(
-                    date_part,
-                    lit(DATE_FORMATTER[date_part](value)),
-                )
-
-        return df
-
-    def _get_partitions(self, write_params: DataFrameReadWriteParams) -> List[str]:
-        date_partition_columns: List[str] = []
-        if write_params.processing_date_partitions:
-            for date_partition in write_params.processing_date_partitions:
-                date_partition_columns.append(date_partition.value)
-
-        # Choose the order of adding partitions based on the `date_partition_first` flag
-        primary_partitions = (
-            date_partition_columns
-            if write_params.processing_date_partitions_first
-            else write_params.column_partitions
-        )
-        secondary_partitions = (
-            write_params.column_partitions
-            if write_params.processing_date_partitions_first
-            else date_partition_columns
-        )
-
-        partitions: List[str] = []
-        if primary_partitions:
-            partitions.extend(primary_partitions)
-        if secondary_partitions:
-            partitions.extend(secondary_partitions)
-
-        return partitions
-
-    def write_dataframe_specific_path(
-        self, df: DataFrame, df_params: OutputDataFrameParams
+    def _write_dataframe_specific_path(
+        self, df: DataFrame, output_df_params: OutputDataFrameParams
     ) -> None:
-        write_params = df_params.df_params
-        bucket_url = self._get_bucket_url(write_params.bucket_params)
-        write_path_url = f"{bucket_url}/{write_params.path_to_dataframe_root}"
+        bucket_url = self._get_bucket_url(output_df_params.dataframe_bucket_params)
+        write_path_url = f"{bucket_url}/{output_df_params.dataframe_specific_path}"
 
-        logger.info(f"Writing DataFrame to path: {write_path_url}...")
-        self.spark_manager.write(
-            env=write_params.bucket_params.env,
-            df=df,
-            path=write_path_url,
-            file_type=df_params.df_params.file_type,
-            partitions=None,
-        )
-
-        if (
-            df_params.write_schema_on_s3
-            or df_params.df_params.file_type != FileType.PARQUET
-        ):
-            self.write_schema(df, write_params)
-
-        logger.info("Dataframe written successfully.\n")
-
-    def write_dataframe(self, df: DataFrame, df_params: OutputDataFrameParams) -> None:
-        write_params = df_params.df_params
-        bucket_url = self._get_bucket_url(write_params.bucket_params)
-        write_path_url = f"{bucket_url}/{write_params.path_to_dataframe_root}"
-        logger.debug(f"Write root path: {write_path_url}")
-
-        partitions = self._get_partitions(write_params)
-        logger.debug(f"Using partitions: {partitions}")
-
-        path = f"{bucket_url}/{self._get_dataframe_path_with_date_partitions(write_params)}"
-
-        # Add the date partitions columns to the DataFrame
-        df_with_date_partitions = df
-        if write_params.processing_date_partitions:
-            for date_partition in write_params.processing_date_partitions:
-                formatted_date_part_value = DATE_FORMATTER[
-                    date_partition.value.replace("part_", "")
-                ](write_params.processing_date)
-
-                df_with_date_partitions = df_with_date_partitions.withColumn(
-                    date_partition.value, lit(formatted_date_part_value)
-                )
-
-        if df_params.overwrite:
+        if output_df_params.overwrite:
             logger.info(
-                f"Overwrite is set to True. Deleting existing objects under path {path}"
+                f"Overwrite is set to True. Deleting existing objects under path {write_path_url}"
             )
             try:
                 self.aws_interface.delete_objects_aws(
-                    write_params.bucket_params.env,
-                    write_params.bucket_params.bucket_name,
-                    path,
+                    env=output_df_params.dataframe_bucket_params.env,
+                    bucket_name=output_df_params.dataframe_bucket_params.bucket_name,
+                    path=output_df_params.dataframe_specific_path,
                 )
             except Exception as e:
                 raise ValueError(f"Error deleting dataframe objects: {e}")
 
             logger.info("Objects deleted successfully.")
 
+        logger.info(f"Writing DataFrame to path: {write_path_url}...")
+        self.spark_manager.write(
+            env=output_df_params.dataframe_bucket_params.env,
+            df=df,
+            path=write_path_url,
+            file_type=output_df_params.dataframe_file_type,
+            partitions=None,
+        )
+
+    def _write_dataframe_base_path(
+        self,
+        df: DataFrame,
+        output_df_params: OutputDataFrameParams,
+    ) -> None:
+        """
+        Writes a DataFrame to a specified location with optional date partitions.
+
+        Args:
+            df (DataFrame): The DataFrame to be written.
+            df_params (OutputDataFrameParams): The parameters for writing the DataFrame.
+
+        Returns:
+            None
+        """
+        bucket_url = self._get_bucket_url(output_df_params.dataframe_bucket_params)
+        write_base_path_url = f"{bucket_url}/{output_df_params.dataframe_base_path}"
+        logger.debug(f"Write base path: {write_base_path_url}")
+
+        df_to_write = df
+        if output_df_params.single_write_date:
+            # Add the date partitions columns to the DataFrame
+            date_partition_path = ""
+            for (
+                date_partition,
+                date_partition_name,
+            ) in (
+                output_df_params.single_write_date.single_write_date_partitions.items()
+            ):
+                date_partition_path += f"{date_partition_name}={date_partition.value}/"
+
+                formatted_date_part_value = DATE_FORMATTER[date_partition.value](
+                    output_df_params.single_write_date.single_write_date
+                )
+
+                df_to_write = df_to_write.withColumn(
+                    date_partition_name, lit(formatted_date_part_value)
+                )
+
+            if output_df_params.overwrite:
+                full_path = f"{write_base_path_url}/{date_partition_path}"
+                logger.info(
+                    f"Overwrite is set to True. Deleting existing objects under path {full_path}"
+                )
+                try:
+                    self.aws_interface.delete_objects_aws(
+                        env=output_df_params.dataframe_bucket_params.env,
+                        bucket_name=output_df_params.dataframe_bucket_params.bucket_name,
+                        path=f"{output_df_params.dataframe_base_path}/{date_partition_path}",
+                    )
+                except Exception as e:
+                    raise ValueError(f"Error deleting dataframe objects: {e}")
+
+                logger.info("Objects deleted successfully.")
+
         logger.info(
-            f"Writing DataFrame to {write_path_url} with partitions {partitions}..."
+            f"Writing DataFrame to base path {write_base_path_url}{'' if not output_df_params.partition_by else f' with partitions {output_df_params.partition_by}'}..."
         )
         self.spark_manager.write(
-            env=write_params.bucket_params.env,
-            df=df_with_date_partitions,
-            path=write_path_url,
-            file_type=df_params.df_params.file_type,
-            partitions=partitions if partitions else None,
+            env=output_df_params.dataframe_bucket_params.env,
+            df=df_to_write,
+            path=write_base_path_url,
+            file_type=output_df_params.dataframe_file_type,
+            partitions=output_df_params.partition_by,
         )
 
         if (
-            df_params.write_schema_on_s3
-            or df_params.df_params.file_type != FileType.PARQUET
+            output_df_params.write_schema_on_s3
+            or output_df_params.dataframe_file_type != FileType.PARQUET
         ):
-            self.write_schema(df, write_params)
+            self.write_schema(df, output_df_params)
+
+        logger.info("Dataframe written successfully.\n")
+
+    def write_dataframe(
+        self,
+        df: DataFrame,
+        output_df_params: OutputDataFrameParams,
+    ) -> None:
+        """
+        Writes a DataFrame to a specified location with optional date partitions.
+
+        Args:
+            df (DataFrame): The DataFrame to be written.
+            df_params (OutputDataFrameParams): The parameters for writing the DataFrame.
+
+        Returns:
+            None
+        """
+        if output_df_params.dataframe_base_path:
+            self._write_dataframe_base_path(df, output_df_params)
+        else:
+            self._write_dataframe_specific_path(df, output_df_params)
+
+        if (
+            output_df_params.write_schema_on_s3
+            or output_df_params.dataframe_file_type != FileType.PARQUET
+        ):
+            self.write_schema(df, output_df_params)
 
         logger.info("Dataframe written successfully.\n")
 
@@ -375,20 +408,28 @@ class DataFrameManager:
 
         for df_name, input_df_params in input_df_params_dict.items():
             logger.info(f"Starting read operation for DataFrame '{df_name}'...")
-            dfs[df_name] = self.read_dataframe(df_params=input_df_params)
-            logger.info(f"Dataframe '{df_name}' read successfully.\n")
+
+            try:
+                dfs[df_name] = self.read_dataframe(
+                    input_df_params=input_df_params, verbose=False
+                )
+                logger.info(f"Dataframe '{df_name}' read successfully.\n")
+            except EmptyDataFrameException:
+                logger.warning(
+                    f"DataFrame '{df_name}' is empty. No data could be found in the specified path(s). Skipping."
+                )
 
         return dfs
 
     def _select_transformation_function(self, transform_params: TransformParams):
         """Select the transformation function based on direct or indirect reference."""
-        if transform_params.fn_direct:
-            return transform_params.fn_direct
+        if transform_params.transform_function:
+            return transform_params.transform_function
         elif transform_params.fn_indirect:
             raise NotImplementedError("Indirect function references are not supported.")
         else:
             raise ValueError(
-                "TransformParams must have either 'fn_direct' or 'fn_indirect' set."
+                "TransformParams must have either 'transform_function' or 'fn_indirect' set."
             )
 
     def apply_transform_to_df(
@@ -450,19 +491,58 @@ class DataFrameManager:
                     f"No OutputDataFrameParams found for dataframe '{df_name}'"
                 )
 
-            if output_df_params.df_params.specific_path:
-                self.write_dataframe_specific_path(df=df, df_params=output_df_params)
-            else:
-                self.write_dataframe(df=df, df_params=output_df_params)
+            self.write_dataframe(df, output_df_params)
+
+    def inject_processing_date_into_step_params(
+        self, step_params: StepParams, processing_date: datetime
+    ) -> None:
+        for _, df_params in step_params.input_dataframes_params.items():
+            if df_params.read_date_params:
+                df_params.read_date_params.was_offset_applied = False
+
+                if df_params.read_date_params.read_dates == "{{processing_date}}":
+                    df_params.read_date_params.read_dates = processing_date
+
+                    offset = df_params.read_date_params.processing_date_offset_days
+                    if offset and offset != 0:
+                        df_params.read_date_params.read_dates
+                        +timedelta(days=offset)
+                        df_params.read_date_params.was_offset_applied = True
+
+        if step_params.output_dataframes_params:
+            for _, df_params in step_params.output_dataframes_params.items():
+                if (
+                    df_params.single_write_date
+                    and df_params.single_write_date.single_write_date
+                    == "{{processing_date}}"
+                ):
+                    df_params.single_write_date.single_write_date = processing_date
 
     def process_step(
         self,
         step_params_json_file_path: Optional[str] = None,
         step_params: Optional[StepParams] = None,
         processing_date: Optional[datetime] = None,
+        write_output_dfs: bool = True,
+        return_output_dfs: bool = False,
     ) -> Optional[DataFrameDict]:
         """
-        Reads the configuration, processes input dataframes, applies transformations, and writes output dataframes.
+        Process a step in the data pipeline.
+
+        Args:
+            step_params_json_file_path (str, optional): The file path to the JSON file containing the step parameters. Either this or `step_params` must be provided. Defaults to None.
+            step_params (StepParams, optional): The step parameters object. Either this or `step_params_json_file_path` must be provided. Defaults to None.
+            processing_date (datetime, optional): The processing date to be injected into the step parameters. Defaults to None.
+            write_output_dfs (bool, optional): Flag indicating whether to write the output DataFrames. Defaults to True.
+            return_output_dfs (bool, optional): Flag indicating whether to return the output DataFrames. This is ignored if `write_output_dfs` is False, in which case it is set to True. Defaults to False.
+
+        Returns:
+            Optional[DataFrameDict]: A dictionary containing the output DataFrames, if `return_output_dfs` is True.
+
+        Raises:
+            ValueError: If neither `step_params_json_file_path` nor `step_params` is provided, or if both are provided.
+            Exception: If an error occurs during the processing step.
+
         """
         if step_params_json_file_path is None and step_params is None:
             raise ValueError(
@@ -485,7 +565,7 @@ class DataFrameManager:
             logger.debug(
                 f"Injecting processing date '{processing_date}' into step parameters..."
             )
-            step_params = self.inject_processing_date_into_step_params(
+            self.inject_processing_date_into_step_params(
                 step_params=step_params,
                 processing_date=processing_date,
             )
@@ -509,74 +589,39 @@ class DataFrameManager:
                 print("No transformations to apply. Moving to Phase 3.")
                 dfs_to_output_dict = input_dfs_dict
 
-            if step_params.output_dataframes_params:
+            if step_params.output_dataframes_params and write_output_dfs:
                 print("\nPhase 3: Write Output DataFrames\n" + "-" * 80)
                 self.write_output_dataframes(
                     dfs_to_output_dict, step_params.output_dataframes_params
                 )
                 logger.info("Wrote all Output DataFrames successfully!")
+
+                if return_output_dfs:
+                    return dfs_to_output_dict
             else:
                 return dfs_to_output_dict
         except Exception as e:
             logger.exception(f"Error during processing step: {e}")
             raise e from None  # Re-raise to propagate the exception
 
-    def inject_processing_date_into_dataframe_params(
-        self,
-        params: Union[DataFrameReadWriteParams, DataFrameReadWriteParams],
-        processing_date: datetime,
-    ) -> Union[DataFrameReadWriteParams, DataFrameReadWriteParams]:
-        modified_params = params
-
-        if params.processing_date == "{{processing_date}}":
-            modified_params.processing_date = processing_date
-        elif params.processing_date == "{{processing_date_previous}}":
-            modified_params.processing_date = processing_date - timedelta(days=1)
-
-        return modified_params
-
-    def inject_processing_date_into_step_params(
-        self, step_params: StepParams, processing_date: datetime
-    ) -> StepParams:
-        modified_step_params = step_params
-
-        for df_name, df_params in modified_step_params.input_dataframes_params.items():
-            new_read_params = self.inject_processing_date_into_dataframe_params(
-                df_params.df_params, processing_date
-            )
-            if not isinstance(new_read_params, DataFrameReadWriteParams):
-                raise ValueError(
-                    f"Expected DataFrameReadParams, but received: {new_read_params}"
-                )
-            modified_step_params.input_dataframes_params[
-                df_name
-            ].df_params = new_read_params
-
-        if modified_step_params.output_dataframes_params:
-            for (
-                df_name,
-                df_params,
-            ) in modified_step_params.output_dataframes_params.items():
-                new_write_params = self.inject_processing_date_into_dataframe_params(
-                    df_params.df_params, processing_date
-                )
-                if not isinstance(new_write_params, DataFrameReadWriteParams):
-                    raise ValueError(
-                        f"Expected DataFrameWriteParams, but received: {new_write_params}"
-                    )
-                modified_step_params.output_dataframes_params[
-                    df_name
-                ].df_params = new_write_params
-
-        return modified_step_params
-
     def process_step_dict(
         self,
         step_params_dict: Union[StepParamsDict, Dict[str, str]],
         processing_date: Optional[datetime] = None,
     ) -> Optional[DataFrameDict]:
         """
-        Reads the configuration, processes input dataframes, applies transformations, and writes output dataframes.
+        Process a dictionary of step parameters.
+
+        Args:
+            step_params_dict (Union[StepParamsDict, Dict[str, str]]): A dictionary containing the step IDs as keys and the corresponding step parameters as values.
+            processing_date (Optional[datetime], optional): The processing date to be used. Defaults to None.
+
+        Returns:
+            Optional[DataFrameDict]: A dictionary containing the processed data frames, if any.
+
+        Raises:
+            Exception: If an error occurs during the processing of any step.
+
         """
         for step_num, (step_id, step_params) in enumerate(
             step_params_dict.items(), start=1
